<?php

namespace Onestartup\Shop\Notifications;

use Illuminate\Bus\Queueable;
use Illuminate\Notifications\Notification;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Notifications\Messages\MailMessage;
use Onestartup\Shop\Model\SaleShop as Order;

use Onestartup\Shop\Model\ClientShop as Client;
use Onestartup\Shop\Model\ShippingAddres as Shipping;
use Onestartup\Shop\Libs\Util;
use Onestartup\Shop\Model\VariableExtra as Variable;

class ResumenOrder extends Notification
{
    use Queueable;

    protected $order;
    protected $products;
    protected $client;
    protected $shipping;
    protected $promo = null;
    protected $additional = null;
    protected $variable = null;

    

    /**
     * Create a new notification instance.
     *
     * @return void
     */
    public function __construct(Order $order, $products, Client $client, $shipping)
    {
        $this->order = $order;
        $this->products = $products;
        $this->shipping = $shipping;
        $this->client = $client;

        $this->additional = $order->info_extra;
        $this->variable = Variable::first();

        if ($this->order->coupon != null) {
            $util = new Util();
            $this->promo = $util->getDiscount($order, $order->coupon);
        }
    }

    /**
     * Get the notification's delivery channels.
     *
     * @param  mixed  $notifiable
     * @return array
     */
    public function via($notifiable)
    {
        return ['mail'];
    }

    /**
     * Get the mail representation of the notification.
     *
     * @param  mixed  $notifiable
     * @return \Illuminate\Notifications\Messages\MailMessage
     */
    public function toMail($notifiable)
    {
        return (new MailMessage)
            ->subject('Resumen de compra')
            ->markdown('shop::mail.resumen.order', [
                'order' => $this->order, 
                'products' => $this->products,
                'client' => $this->client,
                'shipping' => $this->shipping,
                'promo' => $this->promo,
                'additional' => $this->additional,
                'variable' => $this->variable
            ]);
    }

    /**
     * Get the array representation of the notification.
     *
     * @param  mixed  $notifiable
     * @return array
     */
    public function toArray($notifiable)
    {
        return [
            //
        ];
    }
}
